<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of DotClear Picture Show plugin.
# Copyright (c) 2008 Anne-Cécile Calvot. All rights reserved.
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#
# ***** END LICENSE BLOCK *****
/**
 * Gére la configuration utilisée, pour le blog courant, par Pictures Show.
 * */
class psConf extends psFormatsTools {
	public static $all_pictures_picture = 'pf=picturesShow/img/all_pictures.png';
	public static $gallery_picture = 'pf=picturesShow/img/gallery.gif';
	public static $target_link_to_post = 'post_context';
	public static $target_link_to_pstemplate = 'picturesshow_context';
	public static $gallery_theme_gallery = 'gallery';
	public static $gallery_theme_extrai_4 = 'extrai_4';
	public static $gallery_theme_extrai_8 = 'extrai_8';
	public static $gallery_theme_extrai_10 = 'extrai_10';
	public static $gallery_theme_extrai_all = 'extrai_all';
	public static $css_title_blue_silence = 'blue_silence';
	public static $css_title_blowup = 'blowup';

	// whitout maj
	private static $name_space = 'picturesshow';
	private static $name_option_version = 'pictures_show_version';
	private static $name_option_active = 'pictures_show_active';
	private static $name_option_name_rep = 'ps_meta_rep_name';
	private static $name_option_nb_pictures = 'psgal_nb_pictures_per_page';
	private static $name_option_pictures_format_id = 'psgal_pictures_format_id';
	private static $name_option_thumbnails_format_id = 'psgal_thumbnails_format_id';
	private static $name_option_view_menu = 'pstmpl_view_menu';
	private static $name_option_target_link = 'pshome_target_link';
	private static $name_option_picture = 'pshome_picture';
	private static $name_option_display_name = 'pspost_display_name';
	private static $name_option_css_title = 'pspost_css_title';

	private static $version = "0.7.3.2";
	private static $default_meta_rep = "PS_REP";
	private static $default_nb_pictures_per_page = 5;
	private static $default_view_menu = false;
	private static $default_target_link = 'picturesshow_context';
	private static $default_picture = 'pf=picturesShow/img/gallery.gif';
	private static $default_display_name = 'extrai_all';
	private static $default_css_title = 'blowup';

	/**
	 * Permet de savoir si le plugin a besoin d'être initialisé et si oui l'initialise
	 * @return true s'il a été initialisé
	 */
	public static function hasToBeInit() {
		$g_core = &$GLOBALS['core'];
		
		$g_core->blog->settings->setNameSpace(self::$name_space);
		$old_version = $g_core->blog->settings->get(self::$name_option_version);
		$new_version = $g_core->getVersion('picturesShow');

		/* Si la version dans la table (old_version) est égale  à celle du module (new_version), on ne fait rien
		  * si la version dans la table (old_version) est supérieure ou on efface tout avant d'installer 
		  * sinon on efface juste ce qui est périmé */
		if (version_compare($old_version,$new_version,'==')) {
			return false;
		}
		else if ($old_version == null || version_compare($old_version,$new_version,'>')) {
			psConf::cleanAllAndInstall();
		}
		else {
			psConf::cleanOldVersionAndInstall($old_version);
			$g_core->blog->settings->setNameSpace(self::$name_space);
			$g_core->blog->settings->put(self::$name_option_version,$new_version);
		}
		/* */
		return true;  
	}
	
	/**
	 * Initialise les namesspaces et configure le plugin avec les valeurs par defaut.
	 * */     
	private static function intDefaultPlugConf() {
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		$g_core->blog->settings->put(self::$name_option_version,self::$version,'string', 'Pictures Show version');
		$g_core->blog->settings->put(self::$name_option_active,false,'boolean', 'If Pictures Show is activated');
		$g_core->blog->settings->put(self::$name_option_name_rep,self::$default_meta_rep,'string', 'Name of the metadata used by Pictures Show');
	}
	
	/**
	 * Initialise les namesspaces et configure la galerie avec les valeurs par defaut.
	 * */     
	private static function intDefaultGalConf() {
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);

		$g_core->blog->settings->put(self::$name_option_nb_pictures,self::$default_nb_pictures_per_page,'string', 'Number of pictures per page in the gallery');
		$g_core->blog->settings->put(self::$name_option_pictures_format_id,psFormatsTools::$original_pitures_format_id,'string', 'Pictures format id for the gallery');
		$g_core->blog->settings->put(self::$name_option_thumbnails_format_id,psFormatsTools::$original_pitures_format_id,'string', 'Thumbnails format id for the gallery');
		$g_core->blog->settings->put(self::$name_option_view_menu,self::$default_view_menu,'boolean','If the menu is visible in template');
		$g_core->blog->settings->put(self::$name_option_target_link,self::$default_target_link,'string','Target of the link in the home context');
		$g_core->blog->settings->put(self::$name_option_picture,self::$default_picture,'string','Path of the picture in thhe home context');
		$g_core->blog->settings->put(self::$name_option_display_name,self::$default_display_name,'string','Theme for display the gallery in the post context');
		$g_core->blog->settings->put(self::$name_option_css_title,self::$default_css_title,'string','Style to use for display the title "Gallery"');
	}
	
	/**
	 * Efface tout les données concernant PS, initialise les namesspaces et configure le plugin
	 * et la galerie avec les valeurs par defaut.
	 * */     
	public static function cleanAllAndInstall() {
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		foreach ($g_core->blog->settings->dumpSettings() as $k => $v) {
			if ( $v['ns'] == self::$name_space ) {
				$g_core->blog->settings->drop($k);
			}
		}
		# permet de mettre à jour global_settings et local_settings
		$g_core->setBlog($g_core->blog->id);
		self::intDefaultPlugConf();
		self::intDefaultGalConf();
		
		$g_core->setVersion('picturesShow',self::$version);
		$g_core->blog->triggerBlog();
	}
	
	/**
	 * Permet de passer d'une ancienne version à une version plus récente.
	 * */     
	private static function cleanOldVersionAndInstall($old_version) {
		$g_core = &$GLOBALS['core'];
		$int_gal_conf_need = false;
		
		$new_option_nb_pictures_value = null;
		$new_option_pictures_format_id_value = null;
		$new_option_thumbnails_format_id_value = null;
		$new_option_view_menu_value = null;
		$new_option_target_link_value = null;
		$new_option_picture_value = null;
		$new_option_display_name_value = null;
		$new_option_css_title_value = null;
		
		// rq : la version "0.7" n'a jamais été publiée
		if (version_compare($old_version,"0.7",'<=')) {
			/* Si la version existe on garde les paramètres d'affichage et on désactive l'ajoout de tempate */
			$old_name_option_nb_pictures = 'nb_pictures_per_page';
			$old_name_option_gal_format_id = 'gal_pictures_format_id';
			$old_name_option_gal_thumbnails_format_id = 'gal_view_format_id';
			$old_name_option_view_menu = 'ps_view_menu';

			$old_option_nb_pictures_value = $g_core->blog->settings->get($old_name_option_nb_pictures);
			$g_core->blog->settings->drop($old_name_option_nb_pictures);
			$old_option_pictures_format_id_value = $g_core->blog->settings->get($old_name_option_gal_format_id);
			$g_core->blog->settings->drop($old_name_option_gal_format_id);
			$old_option_thumbnails_format_id_value = $g_core->blog->settings->get($old_name_option_gal_thumbnails_format_id);
			$g_core->blog->settings->drop($old_name_option_gal_thumbnails_format_id);
			$old_option_view_menu_value = $g_core->blog->settings->get($old_name_option_view_menu);
			$g_core->blog->settings->drop($old_name_option_view_menu);
			
			// comme 0.7 est la première version les $new_option_XXX_value ne peuvent pas contenir d'autre
			// valeur, inutil d'avoir peur de les écraser inutil d'utiliser if ($old_option_nb_pictures_value != null)
			if ($old_option_nb_pictures_value != null) {
				$new_option_nb_pictures_value = $old_option_nb_pictures_value;
			}
			else {
				$new_option_nb_pictures_value = "-1";
			}
			$new_option_pictures_format_id_value = $old_option_pictures_format_id_value;
			$new_option_thumbnails_format_id_value = $old_option_thumbnails_format_id_value;
			$new_option_view_menu_value = $old_option_view_menu_value;
			$new_option_target_link_value = "none";
			$new_option_display_name_value = "none";
			
			$int_gal_conf_need = true;
		}
		if ( $int_gal_conf_need === true ) {
			# permet de mettre à jour global_settings et local_settings
			$g_core->setBlog($g_core->blog->id);
			
			# création des champs
			self::intDefaultGalConf();
			$g_core->setBlog($g_core->blog->id);
			self::setGalleryConf(	$new_option_nb_pictures_value,
								$new_option_pictures_format_id_value,
								$new_option_thumbnails_format_id_value,
								$new_option_view_menu_value,
								$new_option_target_link_value,
								$new_option_picture_value,
								$new_option_display_name_value,
								$new_option_css_title_value);
								
			$g_core->blog->triggerBlog();
		}
	}

	/**
	 * Configure le plugin.
	 * */
	public static function setPluginConf($active,$name) {
		$g_core = &$GLOBALS['core'];
		$g_core->blog->settings->setNameSpace(self::$name_space);
		$g_core->blog->settings->put(self::$name_option_active,$active,'boolean');
		$g_core->blog->settings->put(self::$name_option_name_rep,$name,'string');
	}

	/**
	 * Configure la galerie.
	 * Tous les arguments vides sont ignorés.
	 *
	 * $nb_pictures : 		Le nombre d'image par page à afficher.
	 * $pictures_format_id : 	Le format d'image principale de la galerie (la cible du lien).
	 * $thumbnails_format_id :	Le format d'image à afficher dans la galerie (la miniature).
	 * $view_menu :		Si le menu s'affiche dans le context picturesShow.
	 * $target_link:		Vers quoi le lien dans le context home pointe.
	 * $picture:			L'image utilisée pour e lien dans le context home.
	 *				N'est pas pris en compte si $picture vaut "none"
	 * $display_name:		Le type d'affichage de la galerie dans le context post.
	 * $css_title:			Le style de theme pris en compte pour l'affichage de la galerie dans post.
	 *				N'est pas pris en compte si $display_name vaut "none"
	 * */     
	public static function setGalleryConf($nb_pictures,$pictures_format_id,$thumbnails_format_id,$view_menu,$target_link,$picture,$display_name,$css_title) {
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		if ($nb_pictures !== null) {
			$g_core->blog->settings->put(self::$name_option_nb_pictures,$nb_pictures,'string');
		}
		if ($pictures_format_id !== null) {
			$g_core->blog->settings->put(self::$name_option_pictures_format_id, $pictures_format_id,'string');
			// permet de changer les valeurs en mémoire et donc de mettre à jour la combo des galleries présente sur la même page
			$g_core->blog->settings->set(self::$name_option_pictures_format_id, $pictures_format_id);
		}
		if ($thumbnails_format_id !== null) {
			$g_core->blog->settings->put(self::$name_option_thumbnails_format_id, $thumbnails_format_id,'string');
			// permet de changer les valeurs en mémoire et donc de mettre à jour la combo des galleries présente sur la même page
			$g_core->blog->settings->set(self::$name_option_thumbnails_format_id, $thumbnails_format_id);
		}
		if ($view_menu !== null) {
			$g_core->blog->settings->put(self::$name_option_view_menu, $view_menu,'boolean');
		}
		if ($target_link !== null) {
			$g_core->blog->settings->put(self::$name_option_target_link,$target_link,'string');
		}
		if ($picture !== null && $target_link != "none") {
			$g_core->blog->settings->put(self::$name_option_picture,$picture,'string');
		}
		if ($display_name !== null) {
			$g_core->blog->settings->put(self::$name_option_display_name,$display_name,'string');
		}
		if ($css_title !== null && $display_name != "none") {
			$g_core->blog->settings->put(self::$name_option_css_title,$css_title,'string');
		}
	}

	public static function getIfActif(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_active);
	}

	public static function getMetaRepName(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_name_rep);
	}

	/**
	 * Peut retourner une chaine vide si on ne veut pas de pagination
	 * */
	public static function getGalNbPicturesPerPage(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_nb_pictures);
	}

	public static function getGalPicturesFormatId(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_pictures_format_id);
	}

	public static function getIfViewMenu(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_view_menu);

	}
	
	public static function getTargetLink(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_target_link);

	}

	public static function getUrlPicture(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_picture);

	}

	public static function getPostGalleryTheme(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_display_name);

	}
	
	public static function getTitleCssStyle(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_css_title);

	}

	public static function getGalThumbnailsFormatId(){
		$g_core = &$GLOBALS['core'];

		$g_core->blog->settings->setNameSpace(self::$name_space);
		return $g_core->blog->settings->get(self::$name_option_thumbnails_format_id);
	}


	/**
	 * Donne l'url du repertoire contenant les différentes galerie du blog courant.
	 * 
	 * $completed Si est à true (valeur par defaut) la méthode retourne l'URL
	 * avec l'host du blog, sinon elle retourne l'URL relative.
	 * */
	public static function getURLPicturesShow($withHost=true){
		return psMedia::getUrlBlogOfRootDir($withHost);
	}

	/**
	 * Donne l'url d'accés aux galeries.
	 * */
	public static function getURLPSGallery(){
		$g_core = &$GLOBALS['core'];
		
		return $g_core->blog->url."picturesShow/";
	}
}
?>
