<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of DotClear Picture Show plugin.
# Copyright (c) 2008 Anne-Cécile Calvot. All rights reserved.
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
#
# ***** END LICENSE BLOCK *****

/**
 * C'est une galerie qui peut être formée de plusieures pages.
 * On ne peut obtenir les images que page par page.
 * On ne peut pas naviguer au seins de la gallerie par contre on peut obtenir des informations sur celle-ci. 
 * */
class psPicturesGallery {
	// Compositor sui posséde la stratégie d'ordonnancement
	protected $scheduler; // <b>psSchedulerPicture</b>

	protected static $the_gallery_id;
	protected static $the_gallery;

	protected $pictures_dir;      	// <b>psPicturesDir</b> les données
	private $viewable_pictures; 	// <b>array{array{psPicture}}</b> "la vue"
	
	// données qui caractérisent les données
	private $relativ_rep;
	
	// données qui caractérisent la vue
	private $pictures_format_id;
	private $thumbnails_format_id;
	private $nb_pictures_per_page;
	
	// antres données
	private $nb_page;
	private $current_page;

	private function nullify() {
		$this->pictures_dir = null;
		$this->viewable_pictures = array();
		$this->viewable_pictures[0]=null;
		$this->nb_page = 0;
		$this->current_page = 0;
		$this->pictures_format_id = '';
	}
	
	protected function loadView() {
		$allPictures=null;
		// cherche toutes les images et les ordonne
		$allPictures = $this->pictures_dir->getAllPictures();
		$allPictures = $this->scheduler->applyOrder($allPictures);
		
		// Fait le tri sur celle qui posséde le format principal et celui de la miniature et organise les restantes par page 
		$this->viewable_pictures = array();
		$nb_pictures = 0;
		$num_page = 0;

		foreach ($allPictures as $picture) {

			if($picture->formatExist($this->pictures_format_id) && $picture->formatExist($this->thumbnails_format_id)) {
				if($nb_pictures == $this->nb_pictures_per_page ) {
					$nb_pictures = 0;
					$num_page++;
				}

				$this->viewable_pictures[$num_page][] = $picture;
				$nb_pictures++;
			}
		}

		if ($nb_pictures!=0 || $num_page!=0) {
			$this->nb_page = $num_page+1;
			$this->setCurrentPageNum($this->current_page);
		}
		else {
			$this->nb_page = 0;
			$this->current_page = 0;
			$this->viewable_pictures[0]=null;
		}
	
	}
	
	/**
	 * Construit le modéle de données et en déduit la vue.
	 *
	 * $scheduler l'objet qui va permettre de trier et d'ordonner les images
	 * $rep_picture nom du repertoire contenant les image qui vont former la galerie
	 * $pictures_format_id :Le format d'image principale de la galerie (la cible du lien).
	 * 			Si l'attribut est null ou si la valeur est fausse, aucune image ne s'affichera.
	 * $current_page	Page courante de la galerie, initialisé à la premiere page de la galerie si non précisée
	 *			Le chiffre doit être compris entre O (inclus) et le nombre de page (exclus).
	 *			Si le numéro de page est null ., initialisé à la premiere page de la galerie si non précisée
	 * $nb_pictures_per_page :	Le nombre d'image par page à afficher.
	 * 				Si l'attribut est null toutes les images seront affichée. 
	 * $thumbnails_format_id :	Le format d'image à afficher dans la galerie (la miniature).
	 * 				Si l'attribut est null ou si Si la valeur est fausse,
	 *				ce sera le format pictures_format_id qui sera choisit. 
	 **/
	protected function __construct($scheduler, $rep_picture,$pictures_format_id,$current_page,$nb_pictures_per_page,$thumbnails_format_id)
	{
		try {
			$this->relativ_rep = $rep_picture;
			$all_formats = fpFFormatLibrary::getAllFormats('psFormat');
			
			if ($pictures_format_id == psFormatsTools::$original_pitures_format_id
				|| psFormatsTools::isFormat($pictures_format_id,$all_formats))
			{
				$this->pictures_format_id= $pictures_format_id;
			}
			else
			{
				$this->nullify();
				return ;
			}
			
			if ($thumbnails_format_id == null ||
					($thumbnails_format_id!=psFormatsTools::$original_pitures_format_id
					 && !psFormatsTools::isFormat($thumbnails_format_id,$all_formats)
					))
			{
				$this->thumbnails_format_id = $this->pictures_format_id;

			}
			else
			{
				$this->thumbnails_format_id = $thumbnails_format_id;
			}

			$this->pictures_dir = psPicturesDir::getPicturesTree($rep_picture,$all_formats);
			$rep_picture = null;

			if ($nb_pictures_per_page == '') {
				$nb_pictures_per_page = -1;
			}
			if ($current_page == '') {
				$current_page = 0;
			}
			$this->nb_pictures_per_page = $nb_pictures_per_page;
			$this->current_page = $current_page;
			$this->scheduler = $scheduler;
			
			
			self::loadView();
		}
		catch (Exception $e) {
			$this->nullify();

			$GLOBALS['core']->error->add(__('This gallery does not exist'.":".$rep_picture));
		}
	}
	
	/******************************************
	 * Méthodes permettant de frabriquer l'objet avec son ordonnanceur.
	 ******************************************/
	
	/**
	 * Permet d'obtenir la galerie $rep_picture en initialisant
	 * la page courante à $page_num.
	 *
	 * $rep_picture nom du repertoire contenant les image qui vont former la galerie
	 *		Si est vide le resultat retourné est null. 
	 * $page_num	Page courante de la galerie, initialisé à la premiere page de la galerie si non précisée
	 *		Le chiffre doit être compris entre O (inclus) et le nombre de page (exclus).
	 *		Si le numéro de page est trop grand ou trop petit c'est respectivement
	 *		la derniere et la premier page qui seront choisie.
	 * $pictures_format_id :Le format d'image principale de la galerie (la cible du lien).
	 * 			Si l'attribut est null, ce sera le format 'original' qui sera choisit. 
	 * 			Si la valeur est fausse, aucune image ne s'affichera.
	 * $thumbnails_format_id :	Le format d'image à afficher dans la galerie (la miniature).
	 * 				Si l'attribut est null, ce sera le format pictures_format_id qui sera choisit. 
	 * 				Si la valeur est fausse, aucune image ne s'affichera.
	 * $nb_pictures_per_page :	Le nombre d'image par page à afficher.
	 * 				Si l'attribut est null ou inférieur à 1, toutes les images seront affichée. 
	 * */
	public static function getGallery($rep_picture,$page_num,$pictures_format_id,$thumbnails_format_id,$nb_pictures_per_page)
	{
		/*
		 * Gestion des arguments
		 */
		if ($rep_picture === null || $rep_picture === "")
		{
			$GLOBALS['core']->error->add(__('Impossible to create the gallery -public-'));
			return null;
		}
		if ($page_num == null)
		{
			$page_num=0;
		}
		if ($pictures_format_id == null)
		{
			$pictures_format_id=psFormatsTools::$original_pitures_format_id;
		}
		
		if ($nb_pictures_per_page < 1 || $nb_pictures_per_page=='') {
			$nb_pictures_per_page='-1';
		}

		/* Pour ne pas avoir a recharger la galerie à chaque fois,
	  	 * on note la dernière galerie chargée.
	  	 * Si on travaille toujours sur la même galerie et
	  	 * que l'on ne demande pas la première page (choix "arbitraire")
	  	 * on ne la recharge pas.
	  	 * */
		$code_gal = 'G'.$rep_picture.$pictures_format_id.$thumbnails_format_id.$nb_pictures_per_page;
		if ($page_num!=0
			&& self::$the_gallery_id == $code_gal)
		{
			self::$the_gallery->setCurrentPageNum($page_num);
		}
		else {
			self::$the_gallery_id = $code_gal;
			$scheduler = new psSchedulerPicture($rep_picture);
			self::$the_gallery = new psPicturesGallery($scheduler,$rep_picture,$pictures_format_id,$page_num,$nb_pictures_per_page,$thumbnails_format_id);
		}

		return self::$the_gallery;
	}
	
	/**
	 * Permet d'obtenir galerie $rep_picture d'une seule page
	 * avec $nb_pictures images choisi alléatoirement dans le repertoire $rep_picture.
	 *
	 * $rep_picture nom du repertoire où l'on va choisir les images
	 * $pictures_format_id :	Le format d'image principale de la galerie (la cible du lien).
	 * 				Si l'attribut est null, ce sera le format 'original' qui sera choisit. 
	 * 				Si la valeur est fausse, aucune image ne s'affichera.
	 * $thumbnails_format_id :	Le format d'image à afficher dans la galerie (la miniature).
	 * 				Si l'attribut est null, ce sera le format pictures_format_id qui sera choisit. 
	 * 				Si la valeur est fausse, aucune image ne s'affichera.
	 * $nb_pictures :	Le nombre d'image à afficher.
	 * 			Si l'attribut est null ou inférieur à 1, toutes les images seront affichée. 
	 * $nb_pictures :	Le nombre d'image min à afficher.
	 * 			Si l'attribut est null ou suppérieur à $nb_pictures, le paramêtre est ignoré. 
	 * */
	public static function getRandomGallery($rep_picture,$pictures_format_id,$thumbnails_format_id,$nb_pictures,$nb_pictures_min)
	{
		/*
		 * Gestion des arguments
		 */
		if ($pictures_format_id == null)
		{
			$pictures_format_id=psFormatsTools::$original_pitures_format_id;
		}
		if ($nb_pictures < 1 || $nb_pictures=='') {
				$nb_pictures = '-1';
		}
		if ( $nb_pictures != '-1' && $nb_pictures_min != null
			&& $nb_pictures_min > $nb_pictures ) {
			$nb_pictures_min = '0';
		}

		/*On recharge toujours la galerie.
		  * */
		$code_gal = 'R'.$rep_picture.$pictures_format_id.$thumbnails_format_id.$nb_pictures;
		self::$the_gallery_id  = $code_gal;
		$scheduler = new psRandomSchedulerPicture($rep_picture,$nb_pictures_min);
		self::$the_gallery = new psPicturesGallery($scheduler,$rep_picture,$pictures_format_id,0,$nb_pictures,$thumbnails_format_id);

		return self::$the_gallery;
	}
	
	/********************
	 * Méthodes outils
	 ********************/
	
	/**
	 * Vérifie qu'il existe un répertoire donnée et qu'il posséde au moins une image qui existe sous la forme
	 * des deux formats pictures_format_id et thumbnails_format_id.
	 *
	 * $rep :	Le nom du répertoire à vérifier.  Le nom du repertoir est un nom relatif par raport au répertoire public.
	 * 		Si l'attribut est null, return false.
	 * $pictures_format_id :	Le 1er type d'image dont on vérifie la présence. (la cible du lien).
	 * 				Si l'attribut est null, ce sera le format 'original' qui sera choisit. 
	 * 				Si la valeur est fausse return false.
	 * $thumbnails_format_id :	Le 2eme type d'image dont on vérifie la présence (la miniature).
	 * 				Si l'attribut est null, ce sera le format pictures_format_id qui sera choisit (ce qui revient à l'ignorer).
	 * 				Si la valeur est fausse, aucune image ne s'affichera.
	 */
	public static function isGallery($rep,$pictures_format_id,$thumbnails_format_id)
	{
		$all_formats = fpFFormatLibrary::getAllFormats('psFormat');
		
		/* Gestion des arguments */
		if ($rep == null) return false;
		if ($pictures_format_id == null)
		{
			$pictures_format_id = psFormatsTools::$original_pitures_format_id;
		}
		elseif ($pictures_format_id != psFormatsTools::$original_pitures_format_id
				&& !psFormatsTools::isFormat($pictures_format_id,$all_formats))
		{
			return false;
		}
		if ($thumbnails_format_id == null)
		{
			$thumbnails_format_id = $pictures_format_id;
		}
		elseif ($thumbnails_format_id != psFormatsTools::$original_pitures_format_id
				&& !psFormatsTools::isFormat($thumbnails_format_id,$all_formats))
		{
			return false;
		}
		
		$scheduler = new psSchedulerPicture($rep);
		$tmp_gal = new psPicturesGallery($scheduler,$rep,$pictures_format_id,0,-1,$thumbnails_format_id);
		
		return ($tmp_gal != null && $tmp_gal->getNbPicture() > 0);
	}
	
	
	/********************
	 * Méthodes d'accés aux données
	 ********************/

	public function getRelativRep() {
		return $this->relativ_rep;
	}

	public function getNbPage( )
	{
		return $this->nb_page;
	}

	public function getCurrentPageNum( )
	{
		return $this->current_page;
	}
	
	/**
	 * Donne le nombre d'images pour la page courante uniquement.
	 * */  
	public function getNbPicture() {
		if (!isset($this->viewable_pictures[0])) return 0;
		if (!isset($this->viewable_pictures[$this->current_page])) return 0;
		
		return count($this->viewable_pictures[$this->current_page]);
	}
	
	/**
	 * Donne les images pour la page courante uniquement.
	 * */
	public function getPicturesOfPage() {
		if (!isset($this->viewable_pictures[0])) return null;
		if (!isset($this->viewable_pictures[$this->current_page])) return null;
		
		return $this->viewable_pictures[$this->current_page];
	}
	
	private function setCurrentPageNum($current_page) {

		if ($current_page > $this->nb_page-1)
		{
			$current_page = $this->nb_page - 1 ;
		}
		elseif ($current_page < 0)
		{
			$current_page = 0 ;
		}
		$this->current_page = $current_page;
	}

	public function getFormatId() {
		return $this->pictures_format_id;
	}

	public function getThumbnailsFormatId() {
		return $this->thumbnails_format_id;
	}

	/**
	 * Donne un tableau indexé permettant de faire une combobox qui liste
	 * l'ensemble des repertoires (pour le blog courant) qui peuvent être
	 * utilisés comme galerie par Pictures Show
	 * (sans gestion des droits et sans selection des images).
	 * */
	private static function getGalleryList() {
		$all_formats = fpFFormatLibrary::getAllFormats('psFormat');
		$dir_name1 = psPicturesTools::getDirNameComboBox(psConf::getGalPicturesFormatId(),false,$all_formats);
		$dir_name2 = psPicturesTools::getDirNameComboBox(psConf::getGalThumbnailsFormatId(),false,$all_formats);
		
		$dirs_combo = array_intersect_key($dir_name1,$dir_name2);
		
		return $dirs_combo;
	}
	
	/**
	 * Donne un tableau indexé permettant de faire une combobox qui liste
	 * l'ensemble des repertoires (pour le blog courant) qui peuvent être
	 * utilisés comme galerie par Pictures Show et pour lesquels l'utilisateur a suffisament de droits.
	 * Les droits nécéssaires sont de pouvoir éditer toutes les images (y compris celles des sous répertoires)
	 * comprises dans la galerie.
	 * Attention la selection des images n'est pas appliquée.
	 * $get_none	<b>boolean</b> Si à true, on rajoute le choix "None" au tableau retourné.
	 *		Le choix "None" qui permet par exemple de ne pas attacher de galerie.
	 * $writable	<b>boolean</b> Par défaut vaut false, si à true on regarde si le contenu est modifiable, 
	 *		sinon juste s'il est visible.
	 * */
	protected static function getEditableGalleryComboBox($get_none,$writable=false) {
		$g_core = &$GLOBALS['core'];
		$dirs_combo = array();
		$dirs_combo_nosort = array();
		
		# on récupère les galeries avec les bons droits
		$dirs_combo_tmp = self::getGalleryList();
		
		# on garde les galeries avec les bons droits
		$my_media = new psMedia($g_core);
		//if (!$g_core->auth->check('media_admin,admin',$g_core->blog->id))
		//{
			foreach( $dirs_combo_tmp as &$dir_name ) {
				
				$my_media->chdir($dir_name);
				$dir_obj = $my_media->ifDirIsEditable($writable);
				if ($dir_obj !== null) {
					$dirs_combo_nosort[$dir_name] = $dir_obj;
				}
			}
		/*}
		else {
			$dirs_combo_nosort = $dirs_combo_tmp;
		
		}*/
		
		# on tri l'affichage
		$my_media->orderDirs($dirs_combo_nosort);
		
		
		# on rajoute none si nécéssaire
		if ($get_none) {
			$dirs_combo["--".__("None")."--"] = '';
		}
		foreach( $dirs_combo_nosort as &$dir_obj ) {
			$dirs_combo[$dir_obj->relname] = $dir_obj->relname;
		}
		return $dirs_combo;
	}

	/**
	 * Donne un tableau indexé permettant de faire une combobox qui liste
	 * l'ensemble des repertoires (pour le blog courant) qui peuvent être
	 * utilisés comme galerie par Pictures Show et pour lesquels l'utilisateur a suffisament de droits.
	 * Attention la selection des images n'est pas appliquée.
	 * $get_none	<b>boolean</b> par defaut à true. Si à true, on rajoute le choix "None" au tableau retourné.
	 *		Le choix "None" qui permet par exemple de ne pas attacher de galerie.
	 * */
	public static function getGalleryComboBox($get_none=true) {
		
		return self::getEditableGalleryComboBox($get_none);
	}
}
?>
